// Load data from CSV
d3.csv(
  'https://gist.githubusercontent.com/SirishaManthena/74352799233a05a3ad0496cbc361e3c1/raw/ecdca534f39a1a2051ee63bb7c979fcb59eb8216/amazon_prime_titles.csv',
).then(function (data) {
  // Structure the data based on the provided categorization
  const categories = {
    name: 'Amazon Prime Titles',
    children: [
      {
        name: 'Movies',
        type: 'category',
        children: [
          { name: 'Comedy', type: 'genre', children: [] },
          {
            name: 'Documentary',
            type: 'genre',
            children: [],
          },
          { name: 'Drama', type: 'genre', children: [] },
        ],
      },
      {
        name: 'TV Shows',
        type: 'category',
        children: [
          { name: 'Kids', type: 'genre', children: [] },
          { name: 'Drama', type: 'genre', children: [] },
          { name: 'Anime', type: 'genre', children: [] },
        ],
      },
    ],
  };

  // Group titles by type (Movies, TV Shows) and genre
  data.forEach(function (title) {
    if (title.type === 'Movie') {
      const movieGenre = title.listed_in
        .split(',')[0]
        .trim();
      const movieItem = {
        name: title.title,
        type: 'title',
      };

      // Assign titles to respective genre under Movies category
      switch (movieGenre) {
        case 'Comedy':
          if (
            categories.children[0].children[0].children
              .length < 3
          ) {
            categories.children[0].children[0].children.push(
              movieItem,
            );
          }
          break;
        case 'Documentary':
          if (
            categories.children[0].children[1].children
              .length < 3
          ) {
            categories.children[0].children[1].children.push(
              movieItem,
            );
          }
          break;
        case 'Drama':
          if (
            categories.children[0].children[2].children
              .length < 3
          ) {
            categories.children[0].children[2].children.push(
              movieItem,
            );
          }
          break;
      }
    } else if (title.type === 'TV Show') {
      const tvShowGenre = title.listed_in
        .split(',')[0]
        .trim();
      const tvShowItem = {
        name: title.title,
        type: 'title',
      };

      // Assign titles to respective genre under TV Shows category
      switch (tvShowGenre) {
        case 'Kids':
          if (
            categories.children[1].children[0].children
              .length < 3
          ) {
            categories.children[1].children[0].children.push(
              tvShowItem,
            );
          }
          break;
        case 'Drama':
          if (
            categories.children[1].children[1].children
              .length < 3
          ) {
            categories.children[1].children[1].children.push(
              tvShowItem,
            );
          }
          break;
        case 'Anime':
          if (
            categories.children[1].children[2].children
              .length < 3
          ) {
            categories.children[1].children[2].children.push(
              tvShowItem,
            );
          }
          break;
      }
    }
  });

  // Set the dimensions and margins of the diagram
  const margin = {
    top: 30,
    right: 250,
    bottom: 30,
    left: 125,
  };
  const width = 780 - margin.left - margin.right; // Adjusted width
  const height = 400 - margin.top - margin.bottom;

  const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom)
    .style('display', 'block')
    .style('margin', '0 auto') // Center horizontally
    .append('g')
    .attr(
      'transform',
      'translate(' + margin.left + ',' + margin.top + ')',
    );

  // Add title to the SVG
  svg
    .append('text')
    .attr('x', width / 2)
    .attr('y', 0 - margin.top / 2)
    .attr('text-anchor', 'middle')
    .style('font-size', '20px')
    .style('text-decoration', 'underline')
    .text('Amazon Prime Titles Tree Graph');

  // Create a tree layout and specify the size
  const tree = d3.tree().size([height, width]);

  // Build the tree
  const root = d3.hierarchy(categories);
  const nodes = tree(root);

  // Add links between nodes
  svg
    .selectAll('.link')
    .data(nodes.links())
    .enter()
    .append('path')
    .attr('class', 'link')
    .attr(
      'd',
      d3
        .linkHorizontal()
        .x(function (d) {
          return d.y;
        })
        .y(function (d) {
          return d.x;
        }),
    );

  // Add nodes
  const node = svg
    .selectAll('.node')
    .data(nodes.descendants())
    .enter()
    .append('g')
    .attr('class', 'node')
    .attr('transform', function (d) {
      return 'translate(' + d.y + ',' + d.x + ')';
    });

  // Add shapes to represent different types
  node
    .append(function (d) {
      if (d.data.type === 'category') {
        return document.createElementNS(
          d3.namespaces.svg,
          'rect',
        );
      } else if (d.data.type === 'genre') {
        return document.createElementNS(
          d3.namespaces.svg,
          'ellipse',
        );
      } else {
        return document.createElementNS(
          d3.namespaces.svg,
          'circle',
        );
      }
    })
    .attr('r', function (d) {
      return d.data.type === 'category'
        ? 7
        : d.data.type === 'genre'
          ? 5
          : 3;
    })
    .attr('width', function (d) {
      return d.data.type === 'category' ? 16 : 0;
    })
    .attr('height', function (d) {
      return d.data.type === 'category' ? 16 : 0;
    })
    .attr('rx', function (d) {
      return d.data.type === 'genre' ? 8 : 0;
    })
    .attr('ry', function (d) {
      return d.data.type === 'genre' ? 4 : 0;
    })
    .style('fill', function (d) {
      return d.data.type === 'category'
        ? 'lightsteelblue'
        : d.data.type === 'genre'
          ? 'lightcoral'
          : 'lightgreen';
    });

  // Add text labels to nodes
  node
    .append('text')
    .attr('dy', '.35em')
    .attr('x', function (d) {
      return d.children ? -13 : 13;
    })
    .style('text-anchor', function (d) {
      return d.children ? 'end' : 'start';
    })
    .text(function (d) {
      if (d.data.type === 'title') {
        return d.data.name.length > 30
          ? d.data.name.slice(0, 30) + '...'
          : d.data.name;
      } else {
        return d.data.name;
      }
    });
});
